@@ -609,11 +609,14 @@ tp_restore_synaptics_touches(struct tp_dispatch *tp,
 	    (tp->nfingers_down == tp->num_slots && nfake_touches == tp->num_slots))
 		return;
 
-	/* Synaptics devices may end touch 2 on BTN_TOOL_TRIPLETAP
-	 * and start it again on the next frame with different coordinates
-	 * (#91352). We search the touches we have, if there is one that has
-	 * just ended despite us being on tripletap, we move it back to
-	 * update.
+	/* Synaptics devices may end touch 2 on transition to/fro
+	 * BTN_TOOL_TRIPLETAP and start it again on the next frame with
+	 * different coordinates (bz#91352, gitlab#434). We search the
+	 * touches we have, if there is one that has just ended despite us
+	 * being on tripletap, we move it back to update.
+	 *
+	 * Note: we only handle the transition from 2 to 3 touches, not the
+	 * other way round (see gitlab#434)
 	 */
 	for (i = 0; i < tp->num_slots; i++) {
 		struct tp_touch *t = tp_get_touch(tp, i);
@@ -642,12 +645,14 @@ tp_process_fake_touches(struct tp_dispatch *tp,
 	    EVDEV_MODEL_SYNAPTICS_SERIAL_TOUCHPAD)
 		tp_restore_synaptics_touches(tp, time);
 
-	/* ALPS touchpads always set 3 slots in the kernel, even
+	/* ALPS serial touchpads always set 3 slots in the kernel, even
 	 * where they support less than that. So we get BTN_TOOL_TRIPLETAP
 	 * but never slot 2 because our slot count is wrong.
 	 * This also means that the third touch falls through the cracks and
 	 * is ignored.
 	 *
+	 * See https://gitlab.freedesktop.org/libinput/libinput/issues/408
+	 *
 	 * All touchpad devices have at least one slot so we only do this
 	 * for 2 touches or higher.
 	 *
@@ -662,7 +667,8 @@ tp_process_fake_touches(struct tp_dispatch *tp,
 	 * For a long explanation of what happens, see
 	 * https://gitlab.freedesktop.org/libevdev/libevdev/merge_requests/19
 	 */
-	if (nfake_touches > 1 && tp->has_mt &&
+	if (tp->device->model_flags & EVDEV_MODEL_ALPS_SERIAL_TOUCHPAD &&
+	    nfake_touches > 1 && tp->has_mt &&
 	    tp->nactive_slots > 0 &&
 	    nfake_touches > tp->nactive_slots &&
 	    tp->nactive_slots < tp->num_slots) {
